# Include existing rsync completions
. /usr/share/bash-completion/completions/rsync

# Include existing ssh completions
. /usr/share/bash-completion/completions/ssh

# Function to add custom user@hostname completions
_custom_hosts()
{
    local cur="${COMP_WORDS[COMP_CWORD]}"
    local hosts=$(cat ~/.achar/hosts)
    COMPREPLY=( $(compgen -W "$hosts" -- "$cur") )
}

# Function to handle extended rsync completions
_extended_rsync_completions()
{
    local cur="${COMP_WORDS[COMP_CWORD]}"

    # Use custom host completions if '@' is detected
    if [[ "$cur" == *@* ]]; then
        _custom_hosts
    else
        # Fallback to default rsync completion
        _rsync
    fi
}

# Extend the existing ssh completion function to include custom hosts
_extended_ssh_completions()
{
    local cur="${COMP_WORDS[COMP_CWORD]}"

    # Use custom host completions if '@' is detected
    if [[ "$cur" == *@* ]]; then
        _custom_hosts
    else
        # Fallback to default ssh completion
        _ssh
    fi
}

# Preserve the original rsync completion function
if declare -F _rsync &>/dev/null; then
    complete -r rsync
    complete -F _extended_rsync_completions rsync
fi

# Preserve the original ssh completion function
if declare -F _ssh &>/dev/null; then
    complete -r ssh
    complete -F _extended_ssh_completions ssh
fi
